@@ -423,19 +423,20 @@ impl server::Group for Rustc {
         group.delimiter.map(|it| it.id).unwrap_or_else(tt::TokenId::unspecified)
     }
 
-    fn set_span(&mut self, _group: &mut Self::Group, _span: Self::Span) {
-        // FIXME handle span
+    fn set_span(&mut self, group: &mut Self::Group, span: Self::Span) {
+        if let Some(delim) = &mut group.delimiter {
+            delim.id = span;
+        }
     }
 
-    fn span_open(&mut self, _group: &Self::Group) -> Self::Span {
-        // FIXME handle span
-        // MySpan(self.span_interner.intern(&MySpanData(group.span_open())))
-        tt::TokenId::unspecified()
+    fn span_open(&mut self, group: &Self::Group) -> Self::Span {
+        // FIXME we only store one `TokenId` for the delimiters
+        group.delimiter.map(|it| it.id).unwrap_or_else(tt::TokenId::unspecified)
     }
 
-    fn span_close(&mut self, _group: &Self::Group) -> Self::Span {
-        // FIXME handle span
-        tt::TokenId::unspecified()
+    fn span_close(&mut self, group: &Self::Group) -> Self::Span {
+        // FIXME we only store one `TokenId` for the delimiters
+        group.delimiter.map(|it| it.id).unwrap_or_else(tt::TokenId::unspecified)
     }
 }
 
@@ -453,33 +454,26 @@ impl server::Punct for Rustc {
     fn spacing(&mut self, punct: Self::Punct) -> bridge::Spacing {
         spacing_to_external(punct.spacing)
     }
-    fn span(&mut self, _punct: Self::Punct) -> Self::Span {
-        // FIXME handle span
-        tt::TokenId::unspecified()
+    fn span(&mut self, punct: Self::Punct) -> Self::Span {
+        punct.id
     }
-    fn with_span(&mut self, punct: Self::Punct, _span: Self::Span) -> Self::Punct {
-        // FIXME handle span
-        punct
+    fn with_span(&mut self, punct: Self::Punct, span: Self::Span) -> Self::Punct {
+        tt::Punct { id: span, ..punct }
     }
 }
 
 impl server::Ident for Rustc {
-    fn new(&mut self, string: &str, _span: Self::Span, _is_raw: bool) -> Self::Ident {
-        IdentId(
-            self.ident_interner.intern(&IdentData(tt::Ident {
-                text: string.into(),
-                id: tt::TokenId::unspecified(),
-            })),
-        )
+    fn new(&mut self, string: &str, span: Self::Span, _is_raw: bool) -> Self::Ident {
+        IdentId(self.ident_interner.intern(&IdentData(tt::Ident { text: string.into(), id: span })))
     }
 
-    fn span(&mut self, _ident: Self::Ident) -> Self::Span {
-        // FIXME handle span
-        tt::TokenId::unspecified()
+    fn span(&mut self, ident: Self::Ident) -> Self::Span {
+        self.ident_interner.get(ident.0).0.id
     }
-    fn with_span(&mut self, ident: Self::Ident, _span: Self::Span) -> Self::Ident {
-        // FIXME handle span
-        ident
+    fn with_span(&mut self, ident: Self::Ident, span: Self::Span) -> Self::Ident {
+        let data = self.ident_interner.get(ident.0);
+        let new = IdentData(tt::Ident { id: span, ..data.0.clone() });
+        IdentId(self.ident_interner.intern(&new))
     }
 }
 
@@ -489,8 +483,8 @@ impl server::Literal for Rustc {
         // They must still be present to be ABI-compatible and work with upstream proc_macro.
         "".to_owned()
     }
-    fn from_str(&mut self, _s: &str) -> Result<Self::Literal, ()> {
-        unimplemented!()
+    fn from_str(&mut self, s: &str) -> Result<Self::Literal, ()> {
+        Ok(Literal { text: s.into(), id: tt::TokenId::unspecified() })
     }
     fn symbol(&mut self, literal: &Self::Literal) -> String {
         literal.text.to_string()
@@ -500,10 +494,9 @@ impl server::Literal for Rustc {
     }
 
     fn integer(&mut self, n: &str) -> Self::Literal {
-        let n = if let Ok(n) = n.parse::<i128>() {
-            n.to_string()
-        } else {
-            n.parse::<u128>().unwrap().to_string()
+        let n = match n.parse::<i128>() {
+            Ok(n) => n.to_string(),
+            Err(_) => n.parse::<u128>().unwrap().to_string(),
         };
         Literal { text: n.into(), id: tt::TokenId::unspecified() }
     }
